#ifndef OPTIM_GENETIC_HEADER_
#   define OPTIM_GENETIC_HEADER_ 
    
    // system include
#   include <vector>
#   include <list>
#   include <algorithm>
#   include <fcntl.h>
    
    // private include    
#   include "optimalgo.h"
#   include "smtoken.h"
#   include "semaphop.h"
#   define RECORD_FILE_NAME "bestpopulation.rec"    
    
    //*****************************************************************************
    // Class Declaration
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        
        class OptimGenetic : public OptimAlgo<IndividualType> {
            
            public :
            
                // Constructor
                OptimGenetic ( Nursery<IndividualType> &nursery, unsigned int populationSize, unsigned int nbMutations, unsigned int nbMaxGenerations
                    , unsigned int nbMaxSteadyGenerations, unsigned int maxSimulatedVectors, unsigned int maxSteadyVectors, unsigned int bestSlice )
                    : pvPopulationSize(populationSize),  pvNbMutations(nbMutations * populationSize / 100),  pvNbMaxGenerations(nbMaxGenerations)
                        ,  pvNbMaxSteadyGenerations(nbMaxSteadyGenerations),  pvGenerationNumber(0),  pvSteadyGenerationNumber(0)
                        ,  pvCloneFound(false),  pvBestSlice(bestSlice),  OptimAlgo<IndividualType> (nursery),  pvVibrato(true),  pvWritten(0)
                        ,  pvInteractive(false),  pvLastReset(0)
                {
                    for ( unsigned int index = 0 ; index < pvPopulationSize ; index++ ) {
                        
                        // put a real individual otherwise sizes are bad
                        IndividualType  doe = this->GetNursery().Create();
                        pvCurrentPopulation.push_back(doe);
                    }
                    
                    //                    FillRandomContents(pvCurrentPopulation);
                    this->pvBestCost = this->BestIndividual(pvCurrentPopulation).Cost();
                    this->MaxSteadyVectors(maxSteadyVectors);
                    this->MaxSimulatedVectors(maxSimulatedVectors);
                }
                
                virtual EString Parameters ()
                {
                    EString result ;
                    
                    result << " Population Size : \t\t\t\t\t\t\t" << (int)pvPopulationSize << "\n";
                    result << " Number of mutations : \t\t\t\t\t\t\t" << (int)pvNbMutations << "\n";
                    result << " Max Generations : \t\t\t\t\t\t\t" << (int)pvNbMaxGenerations << "\n";
                    result << " Max Steady Generations : \t\t\t\t\t\t" << (int)pvNbMaxSteadyGenerations << "\n";
                    result << " Slice of Best Kept : \t\t\t\t\t\t\t" << (int)pvBestSlice << "\n";
                    result << OptimAlgo<IndividualType> ::Parameters();
                    return result ;
                }
                
                // Desctructor
                virtual ~OptimGenetic () {}
                
                // Get & Set
                virtual EAlgoType AlgoType () const
                {
                    return GENETIC_ALGORITHM_ALGO ;
                }
                
                // Methods
                void    InitPopulation (EString fileName = "") ;
                bool    Run (EString fileName = "") ;
                bool    Step (std::vector<IndividualType> &refPopulation, std::vector<IndividualType> &nextPopulation
                    , std::vector<IndividualType> &newPopulation, std::vector<SortUnit> &sortArray) ;
                void    Print () ;
                
                bool SuppressClone () const
                {
                    return pvSuppressClone ;
                }
                
                OptimGenetic &SuppressClone ( bool suppress )
                {
                    pvSuppressClone = suppress ;
                    return *this ;
                }
                
                virtual IndividualType &BestIndividual ()
                {
                    return BestIndividual(pvCurrentPopulation);
                }
                
                bool Vibrato ()
                {
                    return pvVibrato ;
                }
                
                OptimGenetic &Vibrato ( bool vibrato )
                {
                    pvVibrato = vibrato ;
                    return *this ;
                }
                
                OptimGenetic &Interactive ( bool interactive )
                {
                    pvInteractive = interactive ;
                    return *this ;
                }
                
                unsigned int NewPopulationSize ()
                {
                    return pvPopulationSize + pvNbMutations + pvPopulationSize * pvBestSlice / 100 ;
                }
            
            private :
            
                // Attributes
                bool                            pvSuppressClone ;
                
                // Configuration parameters
                unsigned int                    pvNbMutations ;
                unsigned int                    pvPopulationSize ;
                unsigned int                    pvNbMaxGenerations ;
                unsigned int                    pvNbMaxSteadyGenerations ;
                unsigned int                    pvBestSlice ;
                bool                            pvVibrato ;
                
                // Algorithm Information parameters
                unsigned int                    pvGenerationNumber ;
                unsigned int                    pvSteadyGenerationNumber ;
                unsigned int                    pvIndividualBirthRank ;
                bool                            pvCloneFound ;
                
                // Methods
                bool                            FillRandomContents (std::vector<IndividualType> &, unsigned int, unsigned int = 0) ;
                void                            ComputeCost (std::vector<IndividualType> &) ;
                bool                            Hybridation (std::vector<IndividualType> &, std::vector<IndividualType> &, std::vector<SortUnit> &sortArray) ;
                void                            Mutation (std::vector<IndividualType> &, std::vector<IndividualType> &, std::vector<SortUnit> &sortArray) ;
                void                            MutateElem (int lFather, std::vector<IndividualType> &refPopulation, std::vector<IndividualType> &nextPopulation) ;
                void                            Vibrato (std::vector<IndividualType> &, std::vector<IndividualType> &, std::vector<SortUnit> &sortArray) ;
                void                            SurvivalSelection (std::vector<IndividualType> &, std::vector<IndividualType> &
                    , std::vector<IndividualType> &, std::vector<SortUnit> &sortArray) ;
                bool                            IsIndividualAClone (std::vector<IndividualType> &, std::vector<IndividualType> &, unsigned int rank) ;
                bool                            IsIndividualACloneFull (std::vector<IndividualType> &, std::vector<IndividualType> &, unsigned int rank) ;
                bool                            IsIndividualACloneFull (std::vector<IndividualType> &, std::vector<IndividualType> &, IndividualType &) ;
                bool                            IsIndividualAClone (std::vector<IndividualType> &, unsigned int rank) ;
                IndividualType                  &BestIndividual (std::vector<IndividualType> &) ;
                
                // Forbidden methods
                                                OptimGenetic () ;
                                                OptimGenetic (const OptimGenetic &) ;
                OptimGenetic                    &operator= (const OptimGenetic &) ;
                std::vector<IndividualType>     pvCurrentPopulation ;
                std::vector<IndividualType>     pvNextPopulation ;
                std::vector<IndividualType>     pvNewPopulation ;
                IndividualType                  keepType ;
                int                             pvWritten ;
                bool                            pvInteractive ;
                unsigned int                    pvLastReset ;
                double                          pvMeanLess ;
                int                             pvMeanLessSteady ;
                Semaphop                        pvSemaphop ;
        };
    template <class IndividualType,class FeatureType> 
        void OptimGenetic<IndividualType, FeatureType> ::InitPopulation ( EString fileName )
        {
            unsigned int    currentSize = pvPopulationSize ;
            unsigned int    startInit = 0 ;
            int             keepGenerationNumber = pvNbMaxSteadyGenerations ;
            
            // if fileName read init population in a file            
            if ( fileName != "" ) {
                unsigned int    index ;
                EString         line ;
                int             file = _open(fileName, O_RDONLY);
                if ( file > 0 ) {
                    ReadOneFileLine(file, line);
                    startInit = atoi(line.c_str());
                    
                    // if number too small try smaller number
                    unsigned int    bestSlice = pvPopulationSize * pvBestSlice / 100 ;
                    if ( bestSlice > 50 ) {
                        if ( startInit < bestSlice ) 
                            startInit = 0 ;
                    }
                    if ( startInit ) {
                        pvCurrentPopulation.clear();
                        for ( index = 0 ; index < startInit ; index++ ) {
                            pvCurrentPopulation.push_back(this->GetNursery().Create());
                        }
                        for ( index = 0 ; index < startInit ; index++ ) {
                            pvCurrentPopulation [index].Read(file);
                        }
                    }
                    _close(file);
                }
            }
            
            // for presearch diminish number of stable generation
            if ( pvNbMaxSteadyGenerations > pvPopulationSize ) 
                pvNbMaxSteadyGenerations = pvPopulationSize ;
            
            // not already initialized do it
            if ( !startInit ) {
                if ( pvPopulationSize > 50 ) {
                    unsigned int    bestSlice = pvPopulationSize * pvBestSlice / 100 ;
                    if ( bestSlice > 50 ) {
                        startInit = pvPopulationSize = bestSlice ;
                        pvCurrentPopulation.resize(pvPopulationSize);
                        Run(fileName);
                    } else if ( pvPopulationSize > 100 ) {
                        startInit = pvPopulationSize = 50 ;
                        pvCurrentPopulation.resize(pvPopulationSize);
                        Run(fileName);
                    }
                }
            }
            std::cout << "Starting Genetic Algorithm Optimization : " << pvPopulationSize << "\n";
            
            double  lMaxCost = 0 ;
            
            pvSteadyGenerationNumber = 0 ;
            pvNbMaxGenerations += pvGenerationNumber ;
            pvWritten = 0 ;
            pvPopulationSize = currentSize ;
            pvNbMaxSteadyGenerations = keepGenerationNumber ;
            if ( !startInit ) 
                startInit = pvCurrentPopulation.size();
            
            // put missing ones the best at the end
            if ( startInit < pvPopulationSize ) {
                pvNextPopulation.clear();
                unsigned int    index ;
                for ( index = startInit ; index < pvPopulationSize ; index++ ) {
                    pvNextPopulation.push_back(this->GetNursery().Create());
                }
                FillRandomContents(pvNextPopulation, pvPopulationSize - startInit, 0);
                for ( index = 0 ; index < startInit ; index++ ) {
                    pvNextPopulation.push_back(pvCurrentPopulation [index]);
                }
                pvCurrentPopulation = pvNextPopulation ;
            }
        }
        
    template <class IndividualType,class FeatureType> 
        bool OptimGenetic<IndividualType, FeatureType> ::Run ( EString fileName )
        {
            std::cout << "Starting Genetic Algorithm Optimization\n";
            
            double  lMaxCost = 0 ;
            
            // empty fileName if not parameter
            if ( fileName == "" ) {
                char    *fileName = RECORD_FILE_NAME ;
                _close(_open(fileName, O_CREAT | O_RDWR | O_TRUNC, 0666));
            }
            pvWritten = 0 ;
#           if 1
                InitPopulation(fileName);
#           else 
                for ( unsigned int index = 0 ; index < pvPopulationSize ; index++ ) {
                    pvCurrentPopulation [index] = this->GetNursery().Create();
                }
                FillRandomContents(pvCurrentPopulation, pvPopulationSize);
                ComputeCost(pvCurrentPopulation);
                pvBestCost = BestIndividual(pvCurrentPopulation).Cost();
#           endif
            
            int i = 0 ;
            
            pvMeanLessSteady = -1 ;
            
            std::vector<SortUnit>   sortArray ;
            
            // dummy init for new population        
            pvNewPopulation = pvCurrentPopulation ;
            pvNextPopulation = pvCurrentPopulation ;
            
            // do steps
            while ( (pvSteadyGenerationNumber < pvNbMaxSteadyGenerations || pvSteadyGenerationNumber < pvGenerationNumber / 10)
                        && pvGenerationNumber < pvNbMaxGenerations
                        && !pvCloneFound
                        && this->CheckOn() ) {
                Step(pvCurrentPopulation, pvNextPopulation, pvNewPopulation, sortArray);
                Step(pvNewPopulation, pvNextPopulation, pvCurrentPopulation, sortArray);
            }
            return true ;
        }
        
    template <class IndividualType,class FeatureType> 
        bool OptimGenetic<IndividualType, FeatureType> ::Step ( std::vector<IndividualType> &refPopulation
            , std::vector<IndividualType> &nextPopulation, std::vector<IndividualType> &newPopulation, std::vector<SortUnit> &sortArray )
        {
            unsigned int    i ;
            unsigned int    refPop = refPopulation.size();
            unsigned int    nbBest = pvBestSlice * refPop / 100 ;
            
            if ( nbBest < 1 ) 
                nbBest = 1 ;
            
            // all the 10% of max stability reset the lower population to boost diversity
            if ( pvMeanLessSteady > 0 && !((unsigned int)pvMeanLessSteady * 20 <= pvNbMaxSteadyGenerations) && pvMeanLessSteady - pvLastReset > 10 ) {
                if ( refPop - nbBest > 0 ) {
                    FillRandomContents(refPopulation, refPop - nbBest);
                    ComputeCost(refPopulation);
                }
                pvLastReset = pvMeanLessSteady ;
                pvMeanLessSteady = -1 ;
            }
            
            // sort population
            std::vector<SortUnit> ::iterator    sortEnd ;
            
            {
                while ( sortArray.size() < refPop ) 
                    sortArray.push_back(SortUnit());
                for ( i = 0 ; i < refPop ; i++ ) {
                    sortArray [i].Affect(true, i, refPopulation [i].Cost());
                }
                sortEnd = sortArray.begin();
                std::advance(sortEnd, refPop);
                std::sort(sortArray.begin(), sortEnd);
            }
            
            // compute mean of less
            if ( refPop - nbBest > 0 ) {
                double  mean = 0.0 ;
                for ( i = 0 ; i < refPop - nbBest ; i++ ) {
                    mean += sortArray [i].cost ;
                }
                if ( pvMeanLessSteady < 0 ) 
                    pvMeanLess = mean ;
                if ( pvMeanLess < mean ) {
                    pvMeanLess = mean ;
                    pvMeanLessSteady = 0 ;
                } else 
                    pvMeanLessSteady++ ;
            }
            
            // do the job next population must be initialised with a refPopulation size
            nextPopulation.resize(refPopulation.size());
            Hybridation(refPopulation, nextPopulation, sortArray);
            if ( pvSteadyGenerationNumber && Vibrato() ) {
                int nbCreated = RandomValue(0, pvBestSlice * pvPopulationSize / 100);
                if ( nbCreated < 1 ) 
                    nbCreated = 1 ;
                while ( nbCreated-- ) {
                    if ( Vibrato() ) 
                        Vibrato(refPopulation, nextPopulation, sortArray);
#                   if 0
                        IndividualType  newType = this->GetNursery().Create();
                        if ( !(SuppressClone() && IsIndividualACloneFull(refPopulation, nextPopulation, newType)) ) 
                            nextPopulation.push_back(newType);
#                   endif
                }
            }
            Mutation(refPopulation, nextPopulation, sortArray);
            ComputeCost(nextPopulation);
            
            // filter on best elem
            unsigned int    nextPop = nextPopulation.size();
            
            {
                unsigned int    offset = pvBestSlice * refPop / 100 ;
                unsigned int    nbBest ;
                if ( offset > refPop ) 
                    offset = refPop ;
                nbBest = offset ;
                
                // keep best value of elem and mutated 
                for (; offset ; offset-- ) {
                    int indexRef = refPop - offset ;
                    int indexNext = nextPop - offset ;
                    if ( refPopulation [indexRef].Cost() < nextPopulation [indexNext].Cost() ) 
                        refPopulation [indexRef] = nextPopulation [indexNext];
                }
                
                // forget the mutated
                nextPop -= nbBest ;
                nextPopulation.resize(nextPop);
            }
            
            // update sort array
            {
                while ( sortArray.size() < refPop + nextPop ) 
                    sortArray.push_back(SortUnit());
                for ( i = 0 ; i < refPop ; i++ ) {
                    sortArray [i].Affect(true, i, refPopulation [i].Cost());
                }
                for ( i = 0 ; i < nextPop ; i++ ) {
                    sortArray [i + refPop].Affect(false, i, nextPopulation [i].Cost());
                }
                sortEnd = sortArray.begin();
                std::advance(sortEnd, refPop + nextPop);
            }
            
            // sort
            std::sort(sortArray.begin(), sortEnd);
            SurvivalSelection(refPopulation, nextPopulation, newPopulation, sortArray);
            
            // after survival best is last
            double  lBestCost = newPopulation [refPop - 1].Cost();
            
            pvGenerationNumber++ ;
            
            EString         message ;
            unsigned int    bWritten = pvWritten ;
            
            if ( pvInteractive ) {
                while ( pvWritten-- ) 
                    message << '\b';
                message << this->pvBestCost << "(" << (int)pvSteadyGenerationNumber << ")";
                _write(2, message.c_str(), message.length());
            }
            pvWritten = message.length() - bWritten ;
            if ( this->pvBestCost < lBestCost ) {
                this->pvBestCost = lBestCost ;
                if ( !pvInteractive ) {
                    EString message ;
                    message << this->pvBestCost << "(" << (int)pvSteadyGenerationNumber << ")";
                    _write(2, message.c_str(), message.length());
                }
                pvWritten = 0 ;
                pvSteadyGenerationNumber = 0 ;
                
                // store all elem of population in file
                char    *fileName = RECORD_FILE_NAME ;
                int     outFile = _open(fileName, O_CREAT | O_RDWR | O_TRUNC, 0666);
                if ( outFile > 0 ) {
                    unsigned int    popSize = newPopulation.size();
                    unsigned int    index ;
                    EString         res ;
                    res << (int)(newPopulation.size()) << " \n";
                    _write(outFile, res.c_str(), res.length());
                    for ( index = 0 ; index < popSize ; index++ ) {
                        newPopulation [index].Display(outFile);
                    }
                    _close(outFile);
                }
            } else {
                pvSteadyGenerationNumber++ ;
            }
            if ( pvCloneFound ) {
                std::cout << "\nLack of gene diversity in next population\n";
            }
            return true ;
        }
        
    //*****************************************************************************
    // METHOD      : print
    // DESCRIPTION : 
    //               
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        void OptimGenetic<IndividualType, FeatureType> ::Print ()
        {
# if 0
    std::cout << _optim.GENETIC_ALGORITHM << std::endl << "\tPopulationSize=" << _populationSize << "\tNbMutationsPerGeneration=" << _nbMutations
        << std::endl << "\tMaxNbGenerations=" << _nbMaxGenerations << "\tMaxNbSteadyGenerations=" << _nbMaxSteadyGenerations ;
# endif
        }
        
    //*****************************************************************************
    // METHOD      : evaluateGeneration
    // DESCRIPTION :
    //               
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        void OptimGenetic<IndividualType, FeatureType> ::ComputeCost ( std::vector<IndividualType> &population )
        {
            unsigned int    index ;
            unsigned int    popSize = population.size();
            
            // protection
            {
                unsigned int    stock = pvSemaphop.GetStockInfo();
                if ( stock ) 
                    pvSemaphop.GetRessource(stock);
            }
            
            // start population evaluation
            unsigned int    nbEvaluated = 0 ;
            
            for ( index = 0 ; index < popSize ; index++ ) {
                if ( !population [index].Evaluated() ) {
                    population [index].EvaluateThread(&pvSemaphop);
                    nbEvaluated++ ;
                }
            }
            
            // wait for completion
            pvSemaphop.GetRessource(nbEvaluated);
            
            // check values
            for ( index = 0 ; index < population.size() ; index++ ) {
                double  cost = population [index].Cost();
                this->SimulatedVectors(this->SimulatedVectors() + 1);
                if ( cost <= this->pvBestCost ) 
                    this->SteadyVectors(this->SteadyVectors() + 1);
                else 
                    this->SteadyVectors(0);
            }
        }
        
    //*****************************************************************************
    // METHOD      : fillRandomContents
    // DESCRIPTION :
    //               
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        bool OptimGenetic<IndividualType, FeatureType> ::FillRandomContents ( std::vector<IndividualType> &population, unsigned int size
            , unsigned int start )
        {
            const unsigned int  NB_MAX_TRY = maxInt(100, size);
            
            for ( unsigned int ind = start ; ind < size ; ind++ ) {
                bool            isAClone = true ;
                unsigned int    lCloneTestNumber = 0 ;
                population [ind] = this->GetNursery().Create();
                while ( isAClone && lCloneTestNumber < NB_MAX_TRY ) {
                    population [ind].RandomFill();
                    if ( SuppressClone() && IsIndividualAClone(population, ind) ) {
                        lCloneTestNumber++ ;
                    } else {
                        isAClone = false ;
                    }
                }
                if ( isAClone ) {
                    EString pvError = "Lack of gene diversity in inital population";
                    return false ;
                }
            }
            return true ;
        }
        
    //*****************************************************************************
    // METHOD      : hybridation
    // DESCRIPTION :
    //               
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        bool OptimGenetic<IndividualType, FeatureType> ::Hybridation ( std::vector<IndividualType> &refPopulation
            , std::vector<IndividualType> &nextPopulation, std::vector<SortUnit> &sortArray )
        {
            unsigned int    refPop = refPopulation.size();
            
            //            std::cout << " ";
            //   std::cout.flush();
            const int       NB_MAX_TRY = maxInt(100, refPop);
            double          lRandomValue = 0 ;
            int             k = 0 ;
            int             i ;
            
            for ( unsigned int i = 0 ; i < refPop ; i++ ) {
                
                // Try to avoid making an hybrid that is a clone from existing 
                // individual. After N successive attempts, accept the clone.
                bool    isAClone = true ;
                int     lCloneTestNumber = 0 ;
                while ( isAClone && lCloneTestNumber < NB_MAX_TRY ) {
                    
                    // First, select distinct parent from previous generation
                    // Parents are chosen with a probability related to the cost function
                    // Selection of father
                    // select father
                    unsigned int    index ;
                    unsigned int    sum ;
                    unsigned int    lMother = 0 ;
                    unsigned int    lFather = 0 ;
                    lFather = sortArray [this->RankingSelect(RandomValue(0, (refPop * (refPop + 1))))].index ;
                    
                    // select mother
                    index = this->RankingSelect(RandomValue(0, (refPop * (refPop + 1))));
                    {
                        lMother = sortArray [index].index ;
                        if ( lMother == lFather ) {
                            if ( index >= refPop - 1 ) {
                                if ( index > 0 ) 
                                    lMother = index-- ;
                                else {
                                    lCloneTestNumber++ ;
                                    continue ;
                                }
                            } else 
                                lMother = index++ ;
                        }
                    }
                    
                    // Then create hybrid individual with pattern randomly chosen from parents
                    // patterns with probability according to gene efficiency. Parameters are
                    // assumed on different chromosoms, so a different radom variable each
                    nextPopulation [i].MergeFrom(refPopulation [lFather], refPopulation [lMother]);
                    
                    // manage clone
                    if ( SuppressClone() && IsIndividualAClone(refPopulation, nextPopulation, i) ) 
                        lCloneTestNumber++ ;
                    else 
                        isAClone = false ;
                }
                if ( SuppressClone() && isAClone ) {
                    pvCloneFound = true ;
                    std::cout << "Clone found in hybridation (ind=" << i << ")\n";
                    return false ;
                }
            }
            return true ;
        }
        
    //*****************************************************************************
    // METHOD      : mutation
    // DESCRIPTION :
    //               
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        void OptimGenetic<IndividualType, FeatureType> ::MutateElem ( int lFather, std::vector<IndividualType> &refPopulation
            , std::vector<IndividualType> &nextPopulation )
        {
            const int       NB_MAX_TRY = maxInt(100, refPopulation.size());
            unsigned int    lIndIndex ;
            
            // put a copy of it in next set
            IndividualType  newType = refPopulation [lFather];
            
            nextPopulation.push_back(newType);
            lIndIndex = nextPopulation.size() - 1 ;
            
            // mutate this element
            unsigned int    lCloneTestNumber = 0 ;
            bool            isAClone = true ;
            
            while ( isAClone && lCloneTestNumber < (unsigned int)NB_MAX_TRY ) {
                nextPopulation [lIndIndex].Mutate();
                if ( SuppressClone() && IsIndividualACloneFull(refPopulation, nextPopulation, lIndIndex) ) {
                    lCloneTestNumber++ ;
                } else {
                    isAClone = false ;
                }
            }
        }
        
    //*****************************************************************************
    // METHOD      : mutation
    // DESCRIPTION :
    //               
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        void OptimGenetic<IndividualType, FeatureType> ::Mutation ( std::vector<IndividualType> &refPopulation
            , std::vector<IndividualType> &nextPopulation, std::vector<SortUnit> &sortArray )
        {
            unsigned int    refPop = refPopulation.size();
            const int       NB_MAX_TRY = maxInt(100, refPop);
            unsigned int    lIndIndex ;
            
            // empty do nothing
            if ( !nextPopulation.size() ) 
                return ;
            
            int limitBest = refPop - pvBestSlice * refPop / 100 ;
            
            if ( limitBest < 0 ) 
                limitBest = 0 ;
            
            unsigned int    lFather = 0 ;
            
            // Do the indicated mutations on the weakers
            // ---------------------------------
            if ( limitBest > 0 ) 
                for ( unsigned int mut = 0 ; mut < pvNbMutations ; mut++ ) {
                    
                    // Randomly choose the individual index
                    // ------------------------------------
                    unsigned int    index ;
                    unsigned int    sum ;
                    lFather = sortArray [ /* refPop - 1 - */ this->RankingSelect(RandomValue(0, (limitBest * (limitBest + 1))))].index ;
                    
                    // Mutate this elememnt
                    MutateElem(lFather, refPopulation, nextPopulation);
                }
            
            // mutate each of the best 
            for ( lFather = limitBest ; lFather < refPop ; lFather++ ) {
                MutateElem(lFather, refPopulation, nextPopulation);
            }
        }
        
    //*****************************************************************************
    // METHOD      : vibrato
    // DESCRIPTION :
    //               
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        void OptimGenetic<IndividualType, FeatureType> ::Vibrato ( std::vector<IndividualType> &refPopulation
            , std::vector<IndividualType> &nextPopulation, std::vector<SortUnit> &sortArray )
        {
            unsigned int    refPop = refPopulation.size();
            
            // empty do nothing
            if ( !nextPopulation.size() ) 
                return ;
            
            // For each individual to be mutated
            // ---------------------------------
            {
                
                // Randomly choose the individual index
                // ------------------------------------
                int lIndIndex = refPop - pvBestSlice * refPop / 100 ;
                if ( lIndIndex < 0 ) 
                    lIndIndex = 0 ;
                
                // For each param to be mutated
                // ----------------------------
                IndividualType  newType
                    = refPopulation [sortArray [this->RankingSelect(RandomValue((lIndIndex + 1) * (lIndIndex + 2), refPop * (refPop + 1)))].index];
                IndividualType  bestType = newType ;
                FeatureType     oldFeature ;
                FeatureType     newFeature ;
                bool            sthingChanged = false ;
                FeatureType     *tabFeature = new FeatureType [newType.Size()];
                double          bestCost, currCost ;
                bestCost = currCost = refPopulation [sortArray [lIndIndex].index].Cost();
                for ( unsigned int param = 0 ; param < newType.Size() ; param++ ) {
                    IndividualType  tempo = newType ;
                    tempo.Feature(param, (newFeature = tempo.Limits(param).Vibrato(oldFeature = newType.Feature(param))));
                    if ( newFeature != oldFeature && (currCost = tempo.Cost()) > refPopulation [sortArray [lIndIndex].index].Cost() ) {
                        tabFeature [param] = newFeature ;
                        if ( currCost > bestCost ) {
                            bestCost = currCost ;
                            bestType = tempo ;
                        }
                        sthingChanged = true ;
                    } else 
                        tabFeature [param] = oldFeature ;
                }
                if ( sthingChanged ) {
                    for ( unsigned int param = 0 ; param < newType.Size() ; param++ ) {
                        newType.Feature(param, tabFeature [param]);
                    }
                    if ( newType.Cost() < bestCost ) 
                        newType = bestType ;
                }
                if ( sthingChanged
                        && !(SuppressClone() && IsIndividualACloneFull(refPopulation, nextPopulation, newType))
                        && newType.Cost() > refPopulation [sortArray [lIndIndex].index].Cost() ) {
                    refPopulation [sortArray [lIndIndex].index] = newType ;
                }
                delete [] tabFeature ;
            }
        }
        
    //*****************************************************************************
    // METHOD      : survivalSelection
    // DESCRIPTION :
    //               
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        void OptimGenetic<IndividualType, FeatureType> ::SurvivalSelection ( std::vector<IndividualType> &refPopulation
            , std::vector<IndividualType> &nextPopulation, std::vector<IndividualType> &newPopulation, std::vector<SortUnit> &sortArray )
        {
            double          lPrevBestCost = OPTIM_COST_MAX ;
            int             lPrevIndex1 = -1 ;
            int             lPrevIndex2 = -1 ;
            unsigned int    refPop = refPopulation.size();
            unsigned int    nextPop = nextPopulation.size();
            
            //            int                             lTrimIndex = _optim._nbTrimSteps - 1 ;
            // First, define a new population with the best individuals across
            // the 2 populations
            {
                unsigned int    nbIndividuals = refPop + nextPop ;
                unsigned int    i ;
                unsigned int    index ;
                
                // compute the 10% best
                unsigned int    firstSlice = pvPopulationSize * pvBestSlice / 100 ;
                if ( firstSlice < 1 ) 
                    firstSlice = 1 ;
                
                // take the not best as a probability of their rank (sum of rank is n * (n+1) /2 so multiply proba by n*(n+1) and sum by 2
                long    multiplier = (nbIndividuals - firstSlice) * (nbIndividuals - firstSlice + 1);
                for ( i = 0 ; i < pvPopulationSize - firstSlice ; i++ ) {
                    unsigned int    index = 0 ;
                    long            sum = 0 ;
                    unsigned int    indexOrg ;
                    index = this->RankingSelect(RandomValue(0, multiplier));
                    if ( index >= nbIndividuals - firstSlice ) 
                        index = nbIndividuals - firstSlice - 1 ;
                    indexOrg = index ;
                    if ( sortArray [index].selected ) {
                        while ( index < nbIndividuals - firstSlice && sortArray [index].selected ) 
                            index++ ;
                    }
                    if ( index >= nbIndividuals - firstSlice ) {
                        index = indexOrg ;
                        while ( index > 0 && sortArray [index].selected ) 
                            index-- ;
                    }
                    if ( sortArray [index].population ) {
                        newPopulation [i] = refPopulation [sortArray [index].index];
                    } else {
                        newPopulation [i] = nextPopulation [sortArray [index].index];
                    }
                    sortArray [index].selected = true ;
                }
                
                // take the best never taking two with same cost
                index = pvPopulationSize - 1 ;
                double  lastCost = 0 ;
                if ( nbIndividuals > 1 ) 
                    for ( i = nbIndividuals - 1 ; index >= pvPopulationSize - firstSlice && index <= pvPopulationSize - 1 ; i-- ) {
                        if ( sortArray [i].cost != lastCost
                                || index == pvPopulationSize - 1
                                || (sortArray [i].population ? 
                                       refPopulation [sortArray [i].index] != newPopulation [index + 1]
                                    : nextPopulation [sortArray [i].index] != newPopulation [index + 1]) ) {
                            if ( sortArray [i].population ) 
                                newPopulation [index--] = refPopulation [sortArray [i].index];
                            else 
                                newPopulation [index--] = nextPopulation [sortArray [i].index];
                            lastCost = sortArray [i].cost ;
                        }
                    }
                
                // if some are missing fill with low profile : they should die later
                for ( i = 0 ; index >= pvPopulationSize - firstSlice && index <= pvPopulationSize - 1 ; i++ ) {
                    if ( sortArray [i].population ) 
                        newPopulation [index--] = refPopulation [sortArray [i].index];
                    else 
                        newPopulation [index--] = nextPopulation [sortArray [i].index];
                }
            }
        }
        
    //*****************************************************************************
    // METHOD      : isIndividualAClone
    // DESCRIPTION :
    //               
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        bool OptimGenetic<IndividualType, FeatureType> ::IsIndividualAClone ( std::vector<IndividualType> &refPopulation
            , std::vector<IndividualType> &nextPopulation, unsigned int rank )
        {
            unsigned int    refPop = refPopulation.size();
            
            if ( !nextPopulation.size() ) 
                return true ;
            if ( rank > nextPopulation.size() ) 
                rank = nextPopulation.size() - 1 ;
            
            // Test in whole previous population
            // ---------------------------------
            for ( unsigned int i = 0 ; i < refPop ; i++ ) {
                if ( nextPopulation [rank] == refPopulation [i] ) {
                    return true ;
                }
            }
            
            // Test in next population until a given rank
            // ------------------------------------------
            return IsIndividualAClone(nextPopulation, rank);
        }
        
    template <class IndividualType,class FeatureType> 
        bool OptimGenetic<IndividualType, FeatureType> ::IsIndividualAClone ( std::vector<IndividualType> &population, unsigned int rank )
        {
            if ( rank > population.size() ) 
                rank = population.size() - 1 ;
            
            // Test in population until a given rank
            // -------------------------------------
            for ( unsigned int i = 0 ; i < rank ; i++ ) {
                if ( population [i] == population [rank] ) 
                    return true ;
            }
            return false ;
        }
        
    //*****************************************************************************
    // METHOD      : isIndividualACloneFull
    // DESCRIPTION :
    //               
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        bool OptimGenetic<IndividualType, FeatureType> ::IsIndividualACloneFull ( std::vector<IndividualType> &refPopulation
            , std::vector<IndividualType> &nextPopulation, unsigned int rank )
        {
            unsigned int    refPop = refPopulation.size();
            unsigned int    nextPop = nextPopulation.size();
            
            if ( !nextPopulation.size() ) 
                return true ;
            if ( rank > nextPopulation.size() ) 
                rank = nextPopulation.size() - 1 ;
            
            // Test in whole previous population
            // ---------------------------------
            for ( unsigned int i = 0 ; i < refPop ; i++ ) {
                if ( nextPopulation [rank] == refPopulation [i] ) {
                    return true ;
                }
            }
            
            // Test in whole next population (except for a given rank)
            // -------------------------------------------------------
            for ( unsigned int i = 0 ; i < nextPop ; i++ ) {
                if ( i == rank ) 
                    continue ;
                if ( nextPopulation [i] == nextPopulation [rank] ) {
                    return true ;
                }
            }
            return false ;
        }
        
    //*****************************************************************************
    // METHOD      : isIndividualACloneFull
    // DESCRIPTION :
    //               
    //*****************************************************************************
    template <class IndividualType,class FeatureType> 
        bool OptimGenetic<IndividualType, FeatureType> ::IsIndividualACloneFull ( std::vector<IndividualType> &refPopulation
            , std::vector<IndividualType> &nextPopulation, IndividualType &type )
        {
            unsigned int    refPop = refPopulation.size();
            unsigned int    nextPop = nextPopulation.size();
            
            // Test in whole previous population
            // ---------------------------------
            for ( unsigned int i = 0 ; i < refPop ; i++ ) {
                if ( type == refPopulation [i] ) {
                    return true ;
                }
            }
            
            // Test in whole next population
            // ---------------------------------
            for ( unsigned int i = 0 ; i < nextPop ; i++ ) {
                if ( type == nextPopulation [i] ) {
                    return true ;
                }
            }
            return false ;
        }
        
    // MaxCost : get max cost of a population
    template <class IndividualType,class FeatureType> 
        IndividualType &OptimGenetic<IndividualType, FeatureType> ::BestIndividual ( std::vector<IndividualType> &population )
        {
            unsigned int    index ;
            double          bestCost = OPTIM_COST_MIN ;
            unsigned int    indexBest = 0 ;
            unsigned int    size = population.size();
            
            ComputeCost(population);
            for ( index = 0 ; index < size ; index++ ) {
                if ( population [index].Cost() > bestCost ) {
                    indexBest = index ;
                    bestCost = population [index].Cost();
                }
            }
            return population [indexBest];
        }
#endif //OPTIM_GENETIC_HEADER_

